@@ -2,7 +2,6 @@ package commands
 
 import (
 	"fmt"
-	"regexp"
 	"strings"
 
 	"github.com/github/hub/git"
@@ -20,11 +19,15 @@ var cmdCreate = &Command{
 	-p, --private
 		Create a private repository.
 
-	-d, --description=<DESCRIPTION>
-		Use this text as the description of the GitHub repository.
+	-d, --description <DESCRIPTION>
+		A short description of the GitHub repository.
 
-	-h, --homepage=<HOMEPAGE>
-		Use this text as the URL of the GitHub repository.
+	-h, --homepage <HOMEPAGE>
+		A URL with more information about the repository. Use this, for example, if
+		your project has an external website.
+
+	--remote-name <REMOTE>
+		Set the name for the new git remote (default: "origin").
 
 	-o, --browse
 		Open the new repository in a web browser.
@@ -53,22 +56,7 @@ hub-init(1), hub(1)
 `,
 }
 
-var (
-	flagCreatePrivate,
-	flagCreateBrowse,
-	flagCreateCopy bool
-
-	flagCreateDescription,
-	flagCreateHomepage string
-)
-
 func init() {
-	cmdCreate.Flag.BoolVarP(&flagCreatePrivate, "private", "p", false, "PRIVATE")
-	cmdCreate.Flag.BoolVarP(&flagCreateBrowse, "browse", "o", false, "BROWSE")
-	cmdCreate.Flag.BoolVarP(&flagCreateCopy, "copy", "c", false, "COPY")
-	cmdCreate.Flag.StringVarP(&flagCreateDescription, "description", "d", "", "DESCRIPTION")
-	cmdCreate.Flag.StringVarP(&flagCreateHomepage, "homepage", "h", "", "HOMEPAGE")
-
 	CmdRunner.Use(cmdCreate)
 }
 
@@ -85,12 +73,10 @@ func create(command *Command, args *Args) {
 		utils.Check(err)
 		newRepoName = github.SanitizeProjectName(dirName)
 	} else {
-		reg := regexp.MustCompile("^[^-]")
-		if !reg.MatchString(args.FirstParam()) {
-			err = fmt.Errorf("invalid argument: %s", args.FirstParam())
-			utils.Check(err)
-		}
 		newRepoName = args.FirstParam()
+		if newRepoName == "" {
+			utils.Check(command.UsageError(""))
+		}
 	}
 
 	config := github.CurrentConfig()
@@ -109,6 +95,8 @@ func create(command *Command, args *Args) {
 	project := github.NewProject(owner, newRepoName, host.Host)
 	gh := github.NewClient(project.Host)
 
+	flagCreatePrivate := args.Flag.Bool("--private")
+
 	repo, err := gh.Repository(project)
 	if err == nil {
 		foundProject := github.NewProject(repo.FullName, "", project.Host)
@@ -129,6 +117,8 @@ func create(command *Command, args *Args) {
 
 	if repo == nil {
 		if !args.Noop {
+			flagCreateDescription := args.Flag.Value("--description")
+			flagCreateHomepage := args.Flag.Value("--homepage")
 			repo, err := gh.CreateRepository(project, flagCreateDescription, flagCreateHomepage, flagCreatePrivate)
 			utils.Check(err)
 			project = github.NewProject(repo.FullName, "", project.Host)
@@ -138,11 +128,15 @@ func create(command *Command, args *Args) {
 	localRepo, err := github.LocalRepo()
 	utils.Check(err)
 
-	originName := "origin"
+	originName := args.Flag.Value("--remote-name")
+	if originName == "" {
+		originName = "origin"
+	}
+
 	if originRemote, err := localRepo.RemoteByName(originName); err == nil {
 		originProject, err := originRemote.Project()
 		if err != nil || !originProject.SameAs(project) {
-			ui.Errorf(`A git remote named "%s" already exists and is set to push to '%s'.\n`, originRemote.Name, originRemote.PushURL)
+			ui.Errorf("A git remote named '%s' already exists and is set to push to '%s'.\n", originRemote.Name, originRemote.PushURL)
 		}
 	} else {
 		url := project.GitURL("", "", true)
@@ -151,5 +145,7 @@ func create(command *Command, args *Args) {
 
 	webUrl := project.WebURL("", "", "")
 	args.NoForward()
+	flagCreateBrowse := args.Flag.Bool("--browse")
+	flagCreateCopy := args.Flag.Bool("--copy")
 	printBrowseOrCopy(args, webUrl, flagCreateBrowse, flagCreateCopy)
 }
