@@ -20,9 +20,11 @@ var (
 issue [-a <ASSIGNEE>] [-c <CREATOR>] [-@ <USER>] [-s <STATE>] [-f <FORMAT>] [-M <MILESTONE>] [-l <LABELS>] [-d <DATE>] [-o <SORT_KEY> [-^]] [-L <LIMIT>]
 issue show [-f <FORMAT>] <NUMBER>
 issue create [-oc] [-m <MESSAGE>|-F <FILE>] [--edit] [-a <USERS>] [-M <MILESTONE>] [-l <LABELS>]
+issue update <NUMBER> [-m <MESSAGE>|-F <FILE>] [--edit] [-a <USERS>] [-M <MILESTONE>] [-l <LABELS>] [-s <STATE>]
 issue labels [--color]
+issue transfer <NUMBER> <REPO>
 `,
-		Long: `Manage GitHub issues for the current project.
+		Long: `Manage GitHub Issues for the current repository.
 
 ## Commands:
 
@@ -32,28 +34,35 @@ With no arguments, show a list of open issues.
 		Show an existing issue specified by <NUMBER>.
 
 	* _create_:
-		Open an issue in the current project.
+		Open an issue in the current repository.
+
+	* _update_:
+		Update fields of an existing issue specified by <NUMBER>. Use ''--edit''
+		to edit the title and message interactively in the text editor.
 
 	* _labels_:
 		List the labels available in this repository.
 
+	* _transfer_:
+		Transfer an issue to another repository.
+
 ## Options:
-	-a, --assignee=<ASSIGNEE>
-		Display only issues assigned to <ASSIGNEE>.
+	-a, --assignee <ASSIGNEE>
+		In list mode, display only issues assigned to <ASSIGNEE>.
 
-		When opening an issue, this can be a comma-separated list of people to
-		assign to the new issue.
+	-a, --assign <USERS>
+		A comma-separated list of GitHub handles to assign to the created issue.
 
-	-c, --creator=<CREATOR>
+	-c, --creator <CREATOR>
 		Display only issues created by <CREATOR>.
 
-	-@, --mentioned=<USER>
+	-@, --mentioned <USER>
 		Display only issues mentioning <USER>.
 
-	-s, --state=<STATE>
+	-s, --state <STATE>
 		Display issues with state <STATE> (default: "open").
 
-	-f, --format=<FORMAT>
+	-f, --format <FORMAT>
 		Pretty print the contents of the issues using format <FORMAT> (default:
 		"%sC%>(8)%i%Creset  %t%  l%n"). See the "PRETTY FORMATS" section of
 		git-log(1) for some additional details on how placeholders are used in
@@ -109,18 +118,27 @@ With no arguments, show a list of open issues.
 
 		%%: a literal %
 
-	-m, --message=<MESSAGE>
+	--color[=<WHEN>]
+		Enable colored output even if stdout is not a terminal. <WHEN> can be one
+		of "always" (default for ''--color''), "never", or "auto" (default).
+
+	-m, --message <MESSAGE>
 		The text up to the first blank line in <MESSAGE> is treated as the issue
 		title, and the rest is used as issue description in Markdown format.
 
-		If multiple <MESSAGE> options are given, their values are concatenated as
-		separate paragraphs.
+		When multiple ''--message'' are passed, their values are concatenated with a
+		blank line in-between.
 
-	-F, --file=<FILE>
-		Read the issue title and description from <FILE>.
+		When neither ''--message'' nor ''--file'' were supplied to ''issue create'', a
+		text editor will open to author the title and description in.
+
+	-F, --file <FILE>
+		Read the issue title and description from <FILE>. Pass "-" to read from
+		standard input instead. See ''--message'' for the formatting rules.
 
 	-e, --edit
-		Further edit the contents of <FILE> in a text editor before submitting.
+		Open the issue title and description in a text editor before submitting.
+		This can be used in combination with ''--message'' or ''--file''.
 
 	-o, --browse
 		Open the new issue in a web browser.
@@ -128,26 +146,27 @@ With no arguments, show a list of open issues.
 	-c, --copy
 		Put the URL of the new issue to clipboard instead of printing it.
 
-	-M, --milestone=<ID>
-		Display only issues for a GitHub milestone with id <ID>.
+	-M, --milestone <NAME>
+		Display only issues for a GitHub milestone with the name <NAME>.
 
-		When opening an issue, add this issue to a GitHub milestone with id <ID>.
+		When opening an issue, add this issue to a GitHub milestone with the name <NAME>.
+		Passing the milestone number is deprecated.
 
-	-l, --labels=<LABELS>
+	-l, --labels <LABELS>
 		Display only issues with certain labels.
 
 		When opening an issue, add a comma-separated list of labels to this issue.
 
-	-d, --since=<DATE>
+	-d, --since <DATE>
 		Display only issues updated on or after <DATE> in ISO 8601 format.
 
-	-o, --sort=<SORT_KEY>
+	-o, --sort <KEY>
 		Sort displayed issues by "created" (default), "updated" or "comments".
 
 	-^ --sort-ascending
 		Sort by ascending dates instead of descending.
 
-	-L, --limit=<LIMIT>
+	-L, --limit <LIMIT>
 		Display only the first <LIMIT> issues.
 
 	--include-pulls
@@ -155,90 +174,86 @@ With no arguments, show a list of open issues.
 
 	--color
 		Enable colored output for labels list.
+
+## See also:
+
+hub-pr(1), hub(1)
+`,
+		KnownFlags: `
+		-a, --assignee USER
+		-s, --state STATE
+		-f, --format FMT
+		-M, --milestone NAME
+		-c, --creator USER
+		-@, --mentioned USER
+		-l, --labels LIST
+		-d, --since DATE
+		-o, --sort KEY
+		-^, --sort-ascending
+		--include-pulls
+		-L, --limit N
+		--color
 `,
 	}
 
 	cmdCreateIssue = &Command{
-		Key:   "create",
-		Run:   createIssue,
-		Usage: "issue create [-o] [-m <MESSAGE>|-F <FILE>] [-a <USERS>] [-M <MILESTONE>] [-l <LABELS>]",
-		Long:  "Open an issue in the current project.",
+		Key: "create",
+		Run: createIssue,
+		KnownFlags: `
+		-m, --message MSG
+		-F, --file FILE
+		-M, --milestone NAME
+		-l, --labels LIST
+		-a, --assign USER
+		-o, --browse
+		-c, --copy
+		-e, --edit
+`,
 	}
 
 	cmdShowIssue = &Command{
-		Key:   "show",
-		Run:   showIssue,
-		Usage: "issue show <NUMBER>",
-		Long:  "Show an issue in the current project.",
+		Key: "show",
+		Run: showIssue,
+		KnownFlags: `
+		-f, --format FMT
+		--color
+`,
 	}
 
 	cmdLabel = &Command{
-		Key:   "labels",
-		Run:   listLabels,
-		Usage: "issue labels [--color]",
-		Long:  "List the labels available in this repository.",
+		Key: "labels",
+		Run: listLabels,
+		KnownFlags: `
+		--color
+`,
 	}
 
-	flagIssueAssignee,
-	flagIssueState,
-	flagIssueFormat,
-	flagShowIssueFormat,
-	flagIssueMilestoneFilter,
-	flagIssueCreator,
-	flagIssueMentioned,
-	flagIssueLabelsFilter,
-	flagIssueSince,
-	flagIssueSort,
-	flagIssueFile string
+	cmdTransfer = &Command{
+		Key: "transfer",
+		Run: transferIssue,
+	}
 
-	flagIssueMessage messageBlocks
-
-	flagIssueEdit,
-	flagIssueCopy,
-	flagIssueBrowse,
-	flagIssueSortAscending bool
-	flagIssueIncludePulls bool
-
-	flagIssueMilestone uint64
-
-	flagIssueAssignees,
-	flagIssueLabels listFlag
-
-	flagIssueLimit int
-
-	flagLabelsColorize bool
+	cmdUpdate = &Command{
+		Key: "update",
+		Run: updateIssue,
+		KnownFlags: `
+		-m, --message MSG
+		-F, --file FILE
+		-M, --milestone NAME
+		-l, --labels LIST
+		-a, --assign USER
+		-e, --edit
+		-s, --state STATE
+`,
+	}
 )
 
 func init() {
-	cmdShowIssue.Flag.StringVarP(&flagShowIssueFormat, "format", "f", "", "FORMAT")
-
-	cmdCreateIssue.Flag.VarP(&flagIssueMessage, "message", "m", "MESSAGE")
-	cmdCreateIssue.Flag.StringVarP(&flagIssueFile, "file", "F", "", "FILE")
-	cmdCreateIssue.Flag.Uint64VarP(&flagIssueMilestone, "milestone", "M", 0, "MILESTONE")
-	cmdCreateIssue.Flag.VarP(&flagIssueLabels, "label", "l", "LABEL")
-	cmdCreateIssue.Flag.VarP(&flagIssueAssignees, "assign", "a", "ASSIGNEE")
-	cmdCreateIssue.Flag.BoolVarP(&flagIssueBrowse, "browse", "o", false, "BROWSE")
-	cmdCreateIssue.Flag.BoolVarP(&flagIssueCopy, "copy", "c", false, "COPY")
-	cmdCreateIssue.Flag.BoolVarP(&flagIssueEdit, "edit", "e", false, "EDIT")
-
-	cmdIssue.Flag.StringVarP(&flagIssueAssignee, "assignee", "a", "", "ASSIGNEE")
-	cmdIssue.Flag.StringVarP(&flagIssueState, "state", "s", "", "STATE")
-	cmdIssue.Flag.StringVarP(&flagIssueFormat, "format", "f", "%sC%>(8)%i%Creset  %t%  l%n", "FORMAT")
-	cmdIssue.Flag.StringVarP(&flagIssueMilestoneFilter, "milestone", "M", "", "MILESTONE")
-	cmdIssue.Flag.StringVarP(&flagIssueCreator, "creator", "c", "", "CREATOR")
-	cmdIssue.Flag.StringVarP(&flagIssueMentioned, "mentioned", "@", "", "USER")
-	cmdIssue.Flag.StringVarP(&flagIssueLabelsFilter, "labels", "l", "", "LABELS")
-	cmdIssue.Flag.StringVarP(&flagIssueSince, "since", "d", "", "DATE")
-	cmdIssue.Flag.StringVarP(&flagIssueSort, "sort", "o", "created", "SORT_KEY")
-	cmdIssue.Flag.BoolVarP(&flagIssueSortAscending, "sort-ascending", "^", false, "SORT_KEY")
-	cmdIssue.Flag.BoolVarP(&flagIssueIncludePulls, "include-pulls", "", false, "INCLUDE_PULLS")
-	cmdIssue.Flag.IntVarP(&flagIssueLimit, "limit", "L", -1, "LIMIT")
-
-	cmdLabel.Flag.BoolVarP(&flagLabelsColorize, "color", "", false, "COLORIZE")
-
 	cmdIssue.Use(cmdShowIssue)
 	cmdIssue.Use(cmdCreateIssue)
 	cmdIssue.Use(cmdLabel)
+	cmdIssue.Use(cmdTransfer)
+	cmdIssue.Use(cmdUpdate)
 	CmdRunner.Use(cmdIssue)
 }
 
@@ -254,29 +269,47 @@ func listIssues(cmd *Command, args *Args) {
 	if args.Noop {
 		ui.Printf("Would request list of issues for %s\n", project)
 	} else {
-		flagFilters := map[string]string{
-			"state":     flagIssueState,
-			"assignee":  flagIssueAssignee,
-			"milestone": flagIssueMilestoneFilter,
-			"creator":   flagIssueCreator,
-			"mentioned": flagIssueMentioned,
-			"labels":    flagIssueLabelsFilter,
-			"sort":      flagIssueSort,
-		}
 		filters := map[string]interface{}{}
-		for flag, filter := range flagFilters {
-			if cmd.FlagPassed(flag) {
-				filters[flag] = filter
+		if args.Flag.HasReceived("--state") {
+			filters["state"] = args.Flag.Value("--state")
+		}
+		if args.Flag.HasReceived("--assignee") {
+			filters["assignee"] = args.Flag.Value("--assignee")
+		}
+		if args.Flag.HasReceived("--milestone") {
+			milestoneValue := args.Flag.Value("--milestone")
+			if milestoneValue == "none" {
+				filters["milestone"] = milestoneValue
+			} else {
+				milestoneNumber, err := milestoneValueToNumber(milestoneValue, gh, project)
+				utils.Check(err)
+				if milestoneNumber > 0 {
+					filters["milestone"] = milestoneNumber
+				}
 			}
 		}
+		if args.Flag.HasReceived("--creator") {
+			filters["creator"] = args.Flag.Value("--creator")
+		}
+		if args.Flag.HasReceived("--mentioned") {
+			filters["mentioned"] = args.Flag.Value("--mentioned")
+		}
+		if args.Flag.HasReceived("--labels") {
+			labels := commaSeparated(args.Flag.AllValues("--labels"))
+			filters["labels"] = strings.Join(labels, ",")
+		}
+		if args.Flag.HasReceived("--sort") {
+			filters["sort"] = args.Flag.Value("--sort")
+		}
 
-		if flagIssueSortAscending {
+		if args.Flag.Bool("--sort-ascending") {
 			filters["direction"] = "asc"
 		} else {
 			filters["direction"] = "desc"
 		}
 
-		if cmd.FlagPassed("since") {
+		if args.Flag.HasReceived("--since") {
+			flagIssueSince := args.Flag.Value("--since")
 			if sinceTime, err := time.ParseInLocation("2006-01-02", flagIssueSince, time.Local); err == nil {
 				filters["since"] = sinceTime.Format(time.RFC3339)
 			} else {
@@ -284,6 +317,13 @@ func listIssues(cmd *Command, args *Args) {
 			}
 		}
 
+		flagIssueLimit := args.Flag.Int("--limit")
+		flagIssueIncludePulls := args.Flag.Bool("--include-pulls")
+		flagIssueFormat := "%sC%>(8)%i%Creset  %t%  l%n"
+		if args.Flag.HasReceived("--format") {
+			flagIssueFormat = args.Flag.Value("--format")
+		}
+
 		issues, err := gh.FetchIssues(project, filters, flagIssueLimit, func(issue *github.Issue) bool {
 			return issue.PullRequest == nil || flagIssueIncludePulls
 		})
@@ -296,7 +336,7 @@ func listIssues(cmd *Command, args *Args) {
 			}
 		}
 
-		colorize := ui.IsTerminal(os.Stdout)
+		colorize := colorizeOutput(args.Flag.HasReceived("--color"), args.Flag.Value("--color"))
 		for _, issue := range issues {
 			ui.Print(formatIssue(issue, flagIssueFormat, colorize))
 		}
@@ -318,16 +358,13 @@ func formatIssuePlaceholders(issue github.Issue, colorize bool) map[string]strin
 	var labelStrings []string
 	var rawLabels []string
 	for _, label := range issue.Labels {
-		if !colorize {
-			labelStrings = append(labelStrings, fmt.Sprintf(" %s ", label.Name))
-			continue
-		}
-		color, err := utils.NewColor(label.Color)
-		if err != nil {
+		if colorize {
+			color, err := utils.NewColor(label.Color)
 			utils.Check(err)
+			labelStrings = append(labelStrings, colorizeLabel(label, color))
+		} else {
+			labelStrings = append(labelStrings, fmt.Sprintf(" %s ", label.Name))
 		}
-
-		labelStrings = append(labelStrings, colorizeLabel(label, color))
 		rawLabels = append(rawLabels, label.Name)
 	}
 
@@ -390,7 +427,30 @@ func formatIssuePlaceholders(issue github.Issue, colorize bool) map[string]strin
 	}
 }
 
-func formatPullRequestPlaceholders(pr github.PullRequest) map[string]string {
+func formatPullRequestPlaceholders(pr github.PullRequest, colorize bool) map[string]string {
+	prState := pr.State
+	if prState == "open" && pr.Draft {
+		prState = "draft"
+	} else if !pr.MergedAt.IsZero() {
+		prState = "merged"
+	}
+
+	var stateColorSwitch string
+	var prColor int
+	if colorize {
+		switch prState {
+		case "draft":
+			prColor = 37
+		case "merged":
+			prColor = 35
+		case "closed":
+			prColor = 31
+		default:
+			prColor = 32
+		}
+		stateColorSwitch = fmt.Sprintf("\033[%dm", prColor)
+	}
+
 	base := pr.Base.Ref
 	head := pr.Head.Label
 	if pr.IsSameRepo() {
@@ -415,6 +475,8 @@ func formatPullRequestPlaceholders(pr github.PullRequest) map[string]string {
 	}
 
 	return map[string]string{
+		"pS": prState,
+		"pC": stateColorSwitch,
 		"B":  base,
 		"H":  head,
 		"sB": pr.Base.Sha,
@@ -434,9 +496,12 @@ func formatIssue(issue github.Issue, format string, colorize bool) string {
 }
 
 func showIssue(cmd *Command, args *Args) {
-	issueNumber := cmd.Arg(0)
+	issueNumber := ""
+	if args.ParamsSize() > 0 {
+		issueNumber = args.GetParam(0)
+	}
 	if issueNumber == "" {
-		utils.Check(fmt.Errorf(cmd.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 	}
 
 	localRepo, err := github.LocalRepo()
@@ -453,8 +518,9 @@ func showIssue(cmd *Command, args *Args) {
 
 	args.NoForward()
 
-	colorize := ui.IsTerminal(os.Stdout)
-	if flagShowIssueFormat != "" {
+	colorize := colorizeOutput(args.Flag.HasReceived("--color"), args.Flag.Value("--color"))
+	if args.Flag.HasReceived("--format") {
+		flagShowIssueFormat := args.Flag.Value("--format")
 		ui.Print(formatIssue(*issue, flagShowIssueFormat, colorize))
 		return
 	}
@@ -485,8 +551,6 @@ func showIssue(cmd *Command, args *Args) {
 			ui.Printf("\n### comment by @%s on %s\n\n%s\n", comment.User.Login, comment.CreatedAt.String(), comment.Body)
 		}
 	}
-
-	return
 }
 
 func createIssue(cmd *Command, args *Args) {
@@ -508,11 +572,13 @@ func createIssue(cmd *Command, args *Args) {
 Write a message for this issue. The first block of
 text is the title and the rest is the description.`, project))
 
+	flagIssueEdit := args.Flag.Bool("--edit")
+	flagIssueMessage := args.Flag.AllValues("--message")
 	if len(flagIssueMessage) > 0 {
-		messageBuilder.Message = flagIssueMessage.String()
+		messageBuilder.Message = strings.Join(flagIssueMessage, "\n\n")
 		messageBuilder.Edit = flagIssueEdit
-	} else if cmd.FlagPassed("file") {
-		messageBuilder.Message, err = msgFromFile(flagIssueFile)
+	} else if args.Flag.HasReceived("--file") {
+		messageBuilder.Message, err = msgFromFile(args.Flag.Value("--file"))
 		utils.Check(err)
 		messageBuilder.Edit = flagIssueEdit
 	} else {
@@ -541,17 +607,11 @@ text is the title and the rest is the description.`, project))
 		"body":  body,
 	}
 
-	if len(flagIssueLabels) > 0 {
-		params["labels"] = flagIssueLabels
-	}
+	setLabelsFromArgs(params, args)
 
-	if len(flagIssueAssignees) > 0 {
-		params["assignees"] = flagIssueAssignees
-	}
+	setAssigneesFromArgs(params, args)
 
-	if flagIssueMilestone > 0 {
-		params["milestone"] = flagIssueMilestone
-	}
+	setMilestoneFromArgs(params, args, gh, project)
 
 	args.NoForward()
 	if args.Noop {
@@ -560,12 +620,87 @@ text is the title and the rest is the description.`, project))
 		issue, err := gh.CreateIssue(project, params)
 		utils.Check(err)
 
+		flagIssueBrowse := args.Flag.Bool("--browse")
+		flagIssueCopy := args.Flag.Bool("--copy")
 		printBrowseOrCopy(args, issue.HtmlUrl, flagIssueBrowse, flagIssueCopy)
 	}
 
 	messageBuilder.Cleanup()
 }
 
+func updateIssue(cmd *Command, args *Args) {
+	issueNumber := 0
+	if args.ParamsSize() > 0 {
+		issueNumber, _ = strconv.Atoi(args.GetParam(0))
+	}
+	if issueNumber == 0 {
+		utils.Check(cmd.UsageError(""))
+	}
+	if !hasField(args, "--message", "--file", "--labels", "--milestone", "--assign", "--state", "--edit") {
+		utils.Check(cmd.UsageError("please specify fields to update"))
+	}
+
+	localRepo, err := github.LocalRepo()
+	utils.Check(err)
+
+	project, err := localRepo.MainProject()
+	utils.Check(err)
+
+	gh := github.NewClient(project.Host)
+
+	params := map[string]interface{}{}
+	setLabelsFromArgs(params, args)
+	setAssigneesFromArgs(params, args)
+	setMilestoneFromArgs(params, args, gh, project)
+
+	if args.Flag.HasReceived("--state") {
+		params["state"] = args.Flag.Value("--state")
+	}
+
+	if hasField(args, "--message", "--file", "--edit") {
+		messageBuilder := &github.MessageBuilder{
+			Filename: "ISSUE_EDITMSG",
+			Title:    "issue",
+		}
+
+		messageBuilder.AddCommentedSection(fmt.Sprintf(`Editing issue #%d for %s
+
+Update the message for this issue. The first block of
+text is the title and the rest is the description.`, issueNumber, project))
+
+		messageBuilder.Edit = args.Flag.Bool("--edit")
+		flagIssueMessage := args.Flag.AllValues("--message")
+		if len(flagIssueMessage) > 0 {
+			messageBuilder.Message = strings.Join(flagIssueMessage, "\n\n")
+		} else if args.Flag.HasReceived("--file") {
+			messageBuilder.Message, err = msgFromFile(args.Flag.Value("--file"))
+			utils.Check(err)
+		} else {
+			issue, err := gh.FetchIssue(project, strconv.Itoa(issueNumber))
+			utils.Check(err)
+			existingMessage := fmt.Sprintf("%s\n\n%s", issue.Title, issue.Body)
+			messageBuilder.Message = strings.Replace(existingMessage, "\r\n", "\n", -1)
+		}
+
+		title, body, err := messageBuilder.Extract()
+		utils.Check(err)
+		if title == "" {
+			utils.Check(fmt.Errorf("Aborting creation due to empty issue title"))
+		}
+		params["title"] = title
+		params["body"] = body
+		defer messageBuilder.Cleanup()
+	}
+
+	args.NoForward()
+	if args.Noop {
+		ui.Printf("Would update issue #%d for %s\n", issueNumber, project)
+	} else {
+		err := gh.UpdateIssue(project, issueNumber, params)
+		utils.Check(err)
+	}
+}
+
 func listLabels(cmd *Command, args *Args) {
 	localRepo, err := github.LocalRepo()
 	utils.Check(err)
@@ -584,11 +719,66 @@ func listLabels(cmd *Command, args *Args) {
 	labels, err := gh.FetchLabels(project)
 	utils.Check(err)
 
+	flagLabelsColorize := colorizeOutput(args.Flag.HasReceived("--color"), args.Flag.Value("--color"))
 	for _, label := range labels {
 		ui.Print(formatLabel(label, flagLabelsColorize))
 	}
 }
 
+func hasField(args *Args, names ...string) bool {
+	found := false
+	for _, name := range names {
+		if args.Flag.HasReceived(name) {
+			found = true
+		}
+	}
+	return found
+}
+
+func setLabelsFromArgs(params map[string]interface{}, args *Args) {
+	if !args.Flag.HasReceived("--labels") {
+		return
+	}
+	params["labels"] = commaSeparated(args.Flag.AllValues("--labels"))
+}
+
+func setAssigneesFromArgs(params map[string]interface{}, args *Args) {
+	if !args.Flag.HasReceived("--assign") {
+		return
+	}
+	params["assignees"] = commaSeparated(args.Flag.AllValues("--assign"))
+}
+
+func setMilestoneFromArgs(params map[string]interface{}, args *Args, gh *github.Client, project *github.Project) {
+	if !args.Flag.HasReceived("--milestone") {
+		return
+	}
+	milestoneNumber, err := milestoneValueToNumber(args.Flag.Value("--milestone"), gh, project)
+	utils.Check(err)
+	if milestoneNumber == 0 {
+		params["milestone"] = nil
+	} else {
+		params["milestone"] = milestoneNumber
+	}
+}
+
+func colorizeOutput(colorSet bool, when string) bool {
+	if !colorSet || when == "auto" {
+		colorConfig, _ := git.Config("color.ui")
+		switch colorConfig {
+		case "false", "never":
+			return false
+		case "always":
+			return true
+		}
+		return ui.IsTerminal(os.Stdout)
+	} else if when == "never" {
+		return false
+	} else {
+		return true // "always"
+	}
+}
+
 func formatLabel(label github.IssueLabel, colorize bool) string {
 	if colorize {
 		if color, err := utils.NewColor(label.Color); err == nil {
@@ -600,13 +790,136 @@ func formatLabel(label github.IssueLabel, colorize bool) string {
 
 func colorizeLabel(label github.IssueLabel, color *utils.Color) string {
 	bgColorCode := utils.RgbToTermColorCode(color)
-	return fmt.Sprintf("\033[38;5;%d;48;%sm %s \033[m",
-		getSuitableLabelTextColor(color), bgColorCode, label.Name)
+	fgColor := pickHighContrastTextColor(color)
+	fgColorCode := utils.RgbToTermColorCode(fgColor)
+	return fmt.Sprintf("\033[38;%s;48;%sm %s \033[m",
+		fgColorCode, bgColorCode, label.Name)
 }
 
-func getSuitableLabelTextColor(color *utils.Color) int {
-	if color.Brightness() < 0.65 {
-		return 15 // white text
+type contrastCandidate struct {
+	color    *utils.Color
+	contrast float64
+}
+
+func pickHighContrastTextColor(color *utils.Color) *utils.Color {
+	candidates := []contrastCandidate{}
+	appendCandidate := func(c *utils.Color) {
+		candidates = append(candidates, contrastCandidate{
+			color:    c,
+			contrast: color.ContrastRatio(c),
+		})
 	}
-	return 16 // black text
+
+	appendCandidate(utils.White)
+	appendCandidate(utils.Black)
+
+	for _, candidate := range candidates {
+		if candidate.contrast >= 7.0 {
+			return candidate.color
+		}
+	}
+	for _, candidate := range candidates {
+		if candidate.contrast >= 4.5 {
+			return candidate.color
+		}
+	}
+	return utils.Black
+}
+
+func milestoneValueToNumber(value string, client *github.Client, project *github.Project) (int, error) {
+	if value == "" {
+		return 0, nil
+	}
+
+	if milestoneNumber, err := strconv.Atoi(value); err == nil {
+		return milestoneNumber, nil
+	}
+
+	milestones, err := client.FetchMilestones(project)
+	if err != nil {
+		return 0, err
+	}
+	for _, milestone := range milestones {
+		if strings.EqualFold(milestone.Title, value) {
+			return milestone.Number, nil
+		}
+	}
+
+	return 0, fmt.Errorf("error: no milestone found with name '%s'", value)
+}
+
+func transferIssue(cmd *Command, args *Args) {
+	if args.ParamsSize() < 2 {
+		utils.Check(cmd.UsageError(""))
+	}
+
+	localRepo, err := github.LocalRepo()
+	utils.Check(err)
+
+	project, err := localRepo.MainProject()
+	utils.Check(err)
+
+	issueNumber, err := strconv.Atoi(args.GetParam(0))
+	utils.Check(err)
+	targetOwner := project.Owner
+	targetRepo := args.GetParam(1)
+	if strings.Contains(targetRepo, "/") {
+		parts := strings.SplitN(targetRepo, "/", 2)
+		targetOwner = parts[0]
+		targetRepo = parts[1]
+	}
+
+	gh := github.NewClient(project.Host)
+
+	nodeIDsResponse := struct {
+		Source struct {
+			Issue struct {
+				ID string
+			}
+		}
+		Target struct {
+			ID string
+		}
+	}{}
+	err = gh.GraphQL(`
+	query($issue: Int!, $sourceOwner: String!, $sourceRepo: String!, $targetOwner: String!, $targetRepo: String!) {
+		source: repository(owner: $sourceOwner, name: $sourceRepo) {
+			issue(number: $issue) {
+				id
+			}
+		}
+		target: repository(owner: $targetOwner, name: $targetRepo) {
+			id
+		}
+	}`, map[string]interface{}{
+		"issue":       issueNumber,
+		"sourceOwner": project.Owner,
+		"sourceRepo":  project.Name,
+		"targetOwner": targetOwner,
+		"targetRepo":  targetRepo,
+	}, &nodeIDsResponse)
+	utils.Check(err)
+
+	issueResponse := struct {
+		TransferIssue struct {
+			Issue struct {
+				URL string
+			}
+		}
+	}{}
+	err = gh.GraphQL(`
+	mutation($issue: ID!, $repo: ID!) {
+		transferIssue(input: {issueId: $issue, repositoryId: $repo}) {
+			issue {
+				url
+			}
+		}
+	}`, map[string]interface{}{
+		"issue": nodeIDsResponse.Source.Issue.ID,
+		"repo":  nodeIDsResponse.Target.ID,
+	}, &issueResponse)
+	utils.Check(err)
+
+	ui.Println(issueResponse.TransferIssue.Issue.URL)
+	args.NoForward()
 }
