@@ -21,42 +21,46 @@ release [--include-drafts] [--exclude-prereleases] [-L <LIMIT>] [-f <FORMAT>]
 release show [-f <FORMAT>] <TAG>
 release create [-dpoc] [-a <FILE>] [-m <MESSAGE>|-F <FILE>] [-t <TARGET>] <TAG>
 release edit [<options>] <TAG>
-release download <TAG>
+release download <TAG> [-i <PATTERN>]
 release delete <TAG>
 `,
-		Long: `Manage GitHub releases.
+		Long: `Manage GitHub Releases for the current repository.
 
 ## Commands:
 
 With no arguments, shows a list of existing releases.
 
-With '--include-drafts', include draft releases in the listing.
-With '--exclude-prereleases', exclude non-stable releases from the listing.
-
 	* _show_:
 		Show GitHub release notes for <TAG>.
 
-		With '--show-downloads', include the "Downloads" section.
+		With ''--show-downloads'', include the "Downloads" section.
 
 	* _create_:
 		Create a GitHub release for the specified <TAG> name. If git tag <TAG>
-		doesn't exist, it will be created at <TARGET> (default: current branch).
+		does not exist, it will be created at <TARGET> (default: current branch).
 
 	* _edit_:
 		Edit the GitHub release for the specified <TAG> name. Accepts the same
-		options as _create_ command. Publish a draft with '--draft=false'.
+		options as _create_ command. Publish a draft with ''--draft=false''.
 
-		When <MESSAGE> or <FILE> are not specified, a text editor will open
-		pre-populated with current release title and body. To re-use existing title
-		and body unchanged, pass '-m ""'.
+		Without ''--message'' or ''--file'', a text editor will open pre-populated with
+		the current release title and body. To re-use existing title and body
+		unchanged, pass ''-m ""''.
 
 	* _download_:
-	  Download the assets attached to release for the specified <TAG>.
+		Download the assets attached to release for the specified <TAG>.
 
 	* _delete_:
-	  Delete the release and associated assets for the specified <TAG>.
+		Delete the release and associated assets for the specified <TAG>. Note that
+		this does **not** remove the git tag <TAG>.
 
 ## Options:
+	-d, --include-drafts
+		List drafts together with published releases.
+
+	-p, --exclude-prereleases
+		Exclude prereleases from the list.
+
 	-L, --limit
 		Display only the first <LIMIT> releases.
 
@@ -66,24 +70,29 @@ With '--exclude-prereleases', exclude non-stable releases from the listing.
 	-p, --prerelease
 		Create a pre-release.
 
-	-a, --attach=<FILE>
+	-a, --attach <FILE>
 		Attach a file as an asset for this release.
 
-		If <FILE> is in the "<filename>#<text>" format, the text after the '#'
+		If <FILE> is in the "<filename>#<text>" format, the text after the "#"
 		character is taken as asset label.
 
-	-m, --message=<MESSAGE>
+	-m, --message <MESSAGE>
 		The text up to the first blank line in <MESSAGE> is treated as the release
 		title, and the rest is used as release description in Markdown format.
 
-		If multiple <MESSAGE> options are given, their values are concatenated as
-		separate paragraphs.
+		When multiple ''--message'' are passed, their values are concatenated with a
+		blank line in-between.
 
-	-F, --file=<FILE>
-		Read the release title and description from <FILE>.
+		When neither ''--message'' nor ''--file'' were supplied to ''release create'', a
+		text editor will open to author the title and description in.
+
+	-F, --file <FILE>
+		Read the release title and description from <FILE>. Pass "-" to read from
+		standard input instead. See ''--message'' for the formatting rules.
 
 	-e, --edit
-		Further edit the contents of <FILE> in a text editor before submitting.
+		Open the release title and description in a text editor before submitting.
+		This can be used in combination with ''--message'' or ''--file''.
 
 	-o, --browse
 		Open the new release in a web browser.
@@ -91,11 +100,14 @@ With '--exclude-prereleases', exclude non-stable releases from the listing.
 	-c, --copy
 		Put the URL of the new release to clipboard instead of printing it.
 
-	-t, --commitish=<TARGET>
+	-t, --commitish <TARGET>
 		A commit SHA or branch name to attach the release to, only used if <TAG>
-		doesn't already exist (default: main branch).
+		does not already exist (default: main branch).
+	
+	-i, --include <PATTERN>
+		Filter the files in the release to those that match the glob <PATTERN>.
 
-	-f, --format=<FORMAT>
+	-f, --format <FORMAT>
 		Pretty print releases using <FORMAT> (default: "%T%n"). See the "PRETTY
 		FORMATS" section of git-log(1) for some additional details on how
 		placeholders are used in format. The available placeholders for issues are:
@@ -140,88 +152,81 @@ With '--exclude-prereleases', exclude non-stable releases from the listing.
 
 		%%: a literal %
 
+	--color[=<WHEN>]
+		Enable colored output even if stdout is not a terminal. <WHEN> can be one
+		of "always" (default for ''--color''), "never", or "auto" (default).
+
 	<TAG>
 		The git tag name for this release.
 
 ## See also:
 
 hub(1), git-tag(1)
-	`,
+`,
+		KnownFlags: `
+		-d, --include-drafts
+		-p, --exclude-prereleases
+		-L, --limit N
+		-f, --format FMT
+		--color
+`,
 	}
 
 	cmdShowRelease = &Command{
 		Key: "show",
 		Run: showRelease,
+		KnownFlags: `
+		-d, --show-downloads
+		-f, --format FMT
+		--color
+`,
 	}
 
 	cmdCreateRelease = &Command{
 		Key: "create",
 		Run: createRelease,
+		KnownFlags: `
+		-e, --edit
+		-d, --draft
+		-p, --prerelease
+		-o, --browse
+		-c, --copy
+		-a, --attach FILE
+		-m, --message MSG
+		-F, --file FILE
+		-t, --commitish C
+`,
 	}
 
 	cmdEditRelease = &Command{
 		Key: "edit",
 		Run: editRelease,
+		KnownFlags: `
+		-e, --edit
+		-d, --draft
+		-p, --prerelease
+		-a, --attach FILE
+		-m, --message MSG
+		-F, --file FILE
+		-t, --commitish C
+`,
 	}
 
 	cmdDownloadRelease = &Command{
 		Key: "download",
 		Run: downloadRelease,
+		KnownFlags: `
+		-i, --include PATTERN
+		`,
 	}
 
 	cmdDeleteRelease = &Command{
 		Key: "delete",
 		Run: deleteRelease,
 	}
-
-	flagReleaseIncludeDrafts,
-	flagReleaseExcludePrereleases,
-	flagReleaseShowDownloads,
-	flagReleaseDraft,
-	flagReleaseEdit,
-	flagReleaseBrowse,
-	flagReleaseCopy,
-	flagReleasePrerelease bool
-
-	flagReleaseFile,
-	flagReleaseFormat,
-	flagShowReleaseFormat,
-	flagReleaseCommitish string
-
-	flagReleaseMessage messageBlocks
-
-	flagReleaseAssets stringSliceValue
-
-	flagReleaseLimit int
 )
 
 func init() {
-	cmdRelease.Flag.BoolVarP(&flagReleaseIncludeDrafts, "include-drafts", "d", false, "DRAFTS")
-	cmdRelease.Flag.BoolVarP(&flagReleaseExcludePrereleases, "exclude-prereleases", "p", false, "PRERELEASE")
-	cmdRelease.Flag.IntVarP(&flagReleaseLimit, "limit", "L", -1, "LIMIT")
-	cmdRelease.Flag.StringVarP(&flagReleaseFormat, "format", "f", "%T%n", "FORMAT")
-
-	cmdShowRelease.Flag.BoolVarP(&flagReleaseShowDownloads, "show-downloads", "d", false, "DRAFTS")
-	cmdShowRelease.Flag.StringVarP(&flagShowReleaseFormat, "format", "f", "", "FORMAT")
-
-	cmdCreateRelease.Flag.BoolVarP(&flagReleaseEdit, "edit", "e", false, "EDIT")
-	cmdCreateRelease.Flag.BoolVarP(&flagReleaseDraft, "draft", "d", false, "DRAFT")
-	cmdCreateRelease.Flag.BoolVarP(&flagReleasePrerelease, "prerelease", "p", false, "PRERELEASE")
-	cmdCreateRelease.Flag.BoolVarP(&flagReleaseBrowse, "browse", "o", false, "BROWSE")
-	cmdCreateRelease.Flag.BoolVarP(&flagReleaseCopy, "copy", "c", false, "COPY")
-	cmdCreateRelease.Flag.VarP(&flagReleaseAssets, "attach", "a", "ATTACH_ASSETS")
-	cmdCreateRelease.Flag.VarP(&flagReleaseMessage, "message", "m", "MESSAGE")
-	cmdCreateRelease.Flag.StringVarP(&flagReleaseFile, "file", "F", "", "FILE")
-	cmdCreateRelease.Flag.StringVarP(&flagReleaseCommitish, "commitish", "t", "", "COMMITISH")
-
-	cmdEditRelease.Flag.BoolVarP(&flagReleaseEdit, "edit", "e", false, "EDIT")
-	cmdEditRelease.Flag.BoolVarP(&flagReleaseDraft, "draft", "d", false, "DRAFT")
-	cmdEditRelease.Flag.BoolVarP(&flagReleasePrerelease, "prerelease", "p", false, "PRERELEASE")
-	cmdEditRelease.Flag.VarP(&flagReleaseAssets, "attach", "a", "ATTACH_ASSETS")
-	cmdEditRelease.Flag.VarP(&flagReleaseMessage, "message", "m", "MESSAGE")
-	cmdEditRelease.Flag.StringVarP(&flagReleaseFile, "file", "F", "", "FILE")
-	cmdEditRelease.Flag.StringVarP(&flagReleaseCommitish, "commitish", "t", "", "COMMITISH")
-
 	cmdRelease.Use(cmdShowRelease)
 	cmdRelease.Use(cmdCreateRelease)
 	cmdRelease.Use(cmdEditRelease)
@@ -239,6 +244,10 @@ func listReleases(cmd *Command, args *Args) {
 
 	gh := github.NewClient(project.Host)
 
+	flagReleaseLimit := args.Flag.Int("--limit")
+	flagReleaseIncludeDrafts := args.Flag.Bool("--include-drafts")
+	flagReleaseExcludePrereleases := args.Flag.Bool("--exclude-prereleases")
+
 	if args.Noop {
 		ui.Printf("Would request list of releases for %s\n", project)
 	} else {
@@ -248,8 +257,12 @@ func listReleases(cmd *Command, args *Args) {
 		})
 		utils.Check(err)
 
-		colorize := ui.IsTerminal(os.Stdout)
+		colorize := colorizeOutput(args.Flag.HasReceived("--color"), args.Flag.Value("--color"))
 		for _, release := range releases {
+			flagReleaseFormat := "%T%n"
+			if args.Flag.HasReceived("--format") {
+				flagReleaseFormat = args.Flag.Value("--format")
+			}
 			ui.Print(formatRelease(release, flagReleaseFormat, colorize))
 		}
 	}
@@ -313,9 +326,12 @@ func formatRelease(release github.Release, format string, colorize bool) string
 }
 
 func showRelease(cmd *Command, args *Args) {
-	tagName := cmd.Arg(0)
+	tagName := ""
+	if args.ParamsSize() > 0 {
+		tagName = args.GetParam(0)
+	}
 	if tagName == "" {
-		utils.Check(fmt.Errorf(cmdRelease.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 	}
 
 	localRepo, err := github.LocalRepo()
@@ -336,8 +352,8 @@ func showRelease(cmd *Command, args *Args) {
 
 		body := strings.TrimSpace(release.Body)
 
-		colorize := ui.IsTerminal(os.Stdout)
-		if flagShowReleaseFormat != "" {
+		colorize := colorizeOutput(args.Flag.HasReceived("--color"), args.Flag.Value("--color"))
+		if flagShowReleaseFormat := args.Flag.Value("--format"); flagShowReleaseFormat != "" {
 			ui.Print(formatRelease(*release, flagShowReleaseFormat, colorize))
 			return
 		}
@@ -346,7 +362,7 @@ func showRelease(cmd *Command, args *Args) {
 		if body != "" {
 			ui.Printf("\n%s\n", body)
 		}
-		if flagReleaseShowDownloads {
+		if args.Flag.Bool("--show-downloads") {
 			ui.Printf("\n## Downloads\n\n")
 			for _, asset := range release.Assets {
 				ui.Println(asset.DownloadUrl)
@@ -360,9 +376,12 @@ func showRelease(cmd *Command, args *Args) {
 }
 
 func downloadRelease(cmd *Command, args *Args) {
-	tagName := cmd.Arg(0)
+	tagName := ""
+	if args.ParamsSize() > 0 {
+		tagName = args.GetParam(0)
+	}
 	if tagName == "" {
-		utils.Check(fmt.Errorf(cmdRelease.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 	}
 
 	localRepo, err := github.LocalRepo()
@@ -376,12 +395,31 @@ func downloadRelease(cmd *Command, args *Args) {
 	release, err := gh.FetchRelease(project, tagName)
 	utils.Check(err)
 
+	hasPattern := args.Flag.HasReceived("--include")
+	found := false
 	for _, asset := range release.Assets {
+		if hasPattern {
+			isMatch, err := filepath.Match(args.Flag.Value("--include"), asset.Name)
+			utils.Check(err)
+			if !isMatch {
+				continue
+			}
+		}
+
+		found = true
 		ui.Printf("Downloading %s ...\n", asset.Name)
 		err := downloadReleaseAsset(asset, gh)
 		utils.Check(err)
 	}
 
+	if !found && hasPattern {
+		names := []string{}
+		for _, asset := range release.Assets {
+			names = append(names, asset.Name)
+		}
+		utils.Check(fmt.Errorf("the `--include` pattern did not match any available assets:\n%s", strings.Join(names, "\n")))
+	}
+
 	args.NoForward()
 }
 
@@ -406,12 +444,19 @@ func downloadReleaseAsset(asset github.ReleaseAsset, gh *github.Client) (err err
 }
 
 func createRelease(cmd *Command, args *Args) {
-	tagName := cmd.Arg(0)
+	tagName := ""
+	if args.ParamsSize() > 0 {
+		tagName = args.GetParam(0)
+	}
 	if tagName == "" {
-		utils.Check(fmt.Errorf(cmdRelease.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 		return
 	}
 
+	assetsToUpload, close, err := openAssetFiles(args.Flag.AllValues("--attach"))
+	utils.Check(err)
+	defer close()
+
 	localRepo, err := github.LocalRepo()
 	utils.Check(err)
 
@@ -430,13 +475,14 @@ func createRelease(cmd *Command, args *Args) {
 Write a message for this release. The first block of
 text is the title and the rest is the description.`, tagName, project))
 
+	flagReleaseMessage := args.Flag.AllValues("--message")
 	if len(flagReleaseMessage) > 0 {
-		messageBuilder.Message = flagReleaseMessage.String()
-		messageBuilder.Edit = flagReleaseEdit
-	} else if cmd.FlagPassed("file") {
-		messageBuilder.Message, err = msgFromFile(flagReleaseFile)
+		messageBuilder.Message = strings.Join(flagReleaseMessage, "\n\n")
+		messageBuilder.Edit = args.Flag.Bool("--edit")
+	} else if args.Flag.HasReceived("--file") {
+		messageBuilder.Message, err = msgFromFile(args.Flag.Value("--file"))
 		utils.Check(err)
-		messageBuilder.Edit = flagReleaseEdit
+		messageBuilder.Edit = args.Flag.Bool("--edit")
 	} else {
 		messageBuilder.Edit = true
 	}
@@ -450,11 +496,11 @@ text is the title and the rest is the description.`, tagName, project))
 
 	params := &github.Release{
 		TagName:         tagName,
-		TargetCommitish: flagReleaseCommitish,
+		TargetCommitish: args.Flag.Value("--commitish"),
 		Name:            title,
 		Body:            body,
-		Draft:           flagReleaseDraft,
-		Prerelease:      flagReleasePrerelease,
+		Draft:           args.Flag.Bool("--draft"),
+		Prerelease:      args.Flag.Bool("--prerelease"),
 	}
 
 	var release *github.Release
@@ -466,21 +512,48 @@ text is the title and the rest is the description.`, tagName, project))
 		release, err = gh.CreateRelease(project, params)
 		utils.Check(err)
 
+		flagReleaseBrowse := args.Flag.Bool("--browse")
+		flagReleaseCopy := args.Flag.Bool("--copy")
 		printBrowseOrCopy(args, release.HtmlUrl, flagReleaseBrowse, flagReleaseCopy)
 	}
 
 	messageBuilder.Cleanup()
 
-	uploadAssets(gh, release, flagReleaseAssets, args)
+	numAssets := len(assetsToUpload)
+	if numAssets == 0 {
+		return
+	}
+	if args.Noop {
+		ui.Printf("Would attach %d %s\n", numAssets, pluralize(numAssets, "asset"))
+	} else {
+		ui.Errorf("Attaching %d %s...\n", numAssets, pluralize(numAssets, "asset"))
+		uploaded, err := gh.UploadReleaseAssets(release, assetsToUpload)
+		if err != nil {
+			failed := []string{}
+			for _, a := range assetsToUpload[len(uploaded):] {
+				failed = append(failed, fmt.Sprintf("-a %s", a.Name))
+			}
+			ui.Errorf("The release was created, but attaching %d %s failed. ", len(failed), pluralize(len(failed), "asset"))
+			ui.Errorf("You can retry with:\n%s release edit %s -m '' %s\n\n", "hub", release.TagName, strings.Join(failed, " "))
+			utils.Check(err)
+		}
+	}
 }
 
 func editRelease(cmd *Command, args *Args) {
-	tagName := cmd.Arg(0)
+	tagName := ""
+	if args.ParamsSize() > 0 {
+		tagName = args.GetParam(0)
+	}
 	if tagName == "" {
-		utils.Check(fmt.Errorf(cmdRelease.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 		return
 	}
 
+	assetsToUpload, close, err := openAssetFiles(args.Flag.AllValues("--attach"))
+	utils.Check(err)
+	defer close()
+
 	localRepo, err := github.LocalRepo()
 	utils.Check(err)
 
@@ -493,17 +566,14 @@ func editRelease(cmd *Command, args *Args) {
 	utils.Check(err)
 
 	params := map[string]interface{}{}
-
-	if cmd.FlagPassed("commitish") {
-		params["target_commitish"] = flagReleaseCommitish
+	if args.Flag.HasReceived("--commitish") {
+		params["target_commitish"] = args.Flag.Value("--commitish")
 	}
-
-	if cmd.FlagPassed("draft") {
-		params["draft"] = flagReleaseDraft
+	if args.Flag.HasReceived("--draft") {
+		params["draft"] = args.Flag.Bool("--draft")
 	}
-
-	if cmd.FlagPassed("prerelease") {
-		params["prerelease"] = flagReleasePrerelease
+	if args.Flag.HasReceived("--prerelease") {
+		params["prerelease"] = args.Flag.Bool("--prerelease")
 	}
 
 	messageBuilder := &github.MessageBuilder{
@@ -516,22 +586,23 @@ func editRelease(cmd *Command, args *Args) {
 Write a message for this release. The first block of
 text is the title and the rest is the description.`, tagName, project))
 
+	flagReleaseMessage := args.Flag.AllValues("--message")
 	if len(flagReleaseMessage) > 0 {
-		messageBuilder.Message = flagReleaseMessage.String()
-		messageBuilder.Edit = flagReleaseEdit
-	} else if cmd.FlagPassed("file") {
-		messageBuilder.Message, err = msgFromFile(flagReleaseFile)
+		messageBuilder.Message = strings.Join(flagReleaseMessage, "\n\n")
+		messageBuilder.Edit = args.Flag.Bool("--edit")
+	} else if args.Flag.HasReceived("--file") {
+		messageBuilder.Message, err = msgFromFile(args.Flag.Value("--file"))
 		utils.Check(err)
-		messageBuilder.Edit = flagReleaseEdit
+		messageBuilder.Edit = args.Flag.Bool("--edit")
 	} else {
 		messageBuilder.Edit = true
-		messageBuilder.Message = fmt.Sprintf("%s\n\n%s", release.Name, release.Body)
+		messageBuilder.Message = strings.Replace(fmt.Sprintf("%s\n\n%s", release.Name, release.Body), "\r\n", "\n", -1)
 	}
 
 	title, body, err := messageBuilder.Extract()
 	utils.Check(err)
 
-	if title == "" && !cmd.FlagPassed("message") {
+	if title == "" && len(flagReleaseMessage) == 0 {
 		utils.Check(fmt.Errorf("Aborting editing due to empty release title"))
 	}
 
@@ -542,6 +613,7 @@ text is the title and the rest is the description.`, tagName, project))
 		params["body"] = body
 	}
 
+	args.NoForward()
 	if len(params) > 0 {
 		if args.Noop {
 			ui.Printf("Would edit release `%s'\n", tagName)
@@ -553,14 +625,33 @@ text is the title and the rest is the description.`, tagName, project))
 		messageBuilder.Cleanup()
 	}
 
-	uploadAssets(gh, release, flagReleaseAssets, args)
-	args.NoForward()
+	numAssets := len(assetsToUpload)
+	if numAssets == 0 {
+		return
+	}
+	if args.Noop {
+		ui.Printf("Would attach %d %s\n", numAssets, pluralize(numAssets, "asset"))
+	} else {
+		ui.Errorf("Attaching %d %s...\n", numAssets, pluralize(numAssets, "asset"))
+		uploaded, err := gh.UploadReleaseAssets(release, assetsToUpload)
+		if err != nil {
+			failed := []string{}
+			for _, a := range assetsToUpload[len(uploaded):] {
+				failed = append(failed, a.Name)
+			}
+			ui.Errorf("Attaching these assets failed:\n%s\n\n", strings.Join(failed, "\n"))
+			utils.Check(err)
+		}
+	}
 }
 
 func deleteRelease(cmd *Command, args *Args) {
-	tagName := cmd.Arg(0)
+	tagName := ""
+	if args.ParamsSize() > 0 {
+		tagName = args.GetParam(0)
+	}
 	if tagName == "" {
-		utils.Check(fmt.Errorf(cmdRelease.Synopsis()))
+		utils.Check(cmd.UsageError(""))
 		return
 	}
 
@@ -586,32 +677,48 @@ func deleteRelease(cmd *Command, args *Args) {
 	args.NoForward()
 }
 
-func uploadAssets(gh *github.Client, release *github.Release, assets []string, args *Args) {
-	for _, asset := range assets {
+func openAssetFiles(args []string) ([]github.LocalAsset, func(), error) {
+	assets := []github.LocalAsset{}
+	files := []*os.File{}
+
+	for _, arg := range args {
 		var label string
-		parts := strings.SplitN(asset, "#", 2)
-		asset = parts[0]
+		parts := strings.SplitN(arg, "#", 2)
+		path := parts[0]
 		if len(parts) > 1 {
 			label = parts[1]
 		}
 
-		if args.Noop {
-			if label == "" {
-				ui.Errorf("Would attach release asset `%s'\n", asset)
-			} else {
-				ui.Errorf("Would attach release asset `%s' with label `%s'\n", asset, label)
-			}
-		} else {
-			for _, existingAsset := range release.Assets {
-				if existingAsset.Name == filepath.Base(asset) {
-					err := gh.DeleteReleaseAsset(&existingAsset)
-					utils.Check(err)
-					break
-				}
-			}
-			ui.Errorf("Attaching release asset `%s'...\n", asset)
-			_, err := gh.UploadReleaseAsset(release, asset, label)
-			utils.Check(err)
+		file, err := os.Open(path)
+		if err != nil {
+			return nil, nil, err
+		}
+		stat, err := file.Stat()
+		if err != nil {
+			return nil, nil, err
+		}
+		files = append(files, file)
+
+		assets = append(assets, github.LocalAsset{
+			Name:     path,
+			Label:    label,
+			Size:     stat.Size(),
+			Contents: file,
+		})
+	}
+
+	close := func() {
+		for _, f := range files {
+			f.Close()
 		}
 	}
+
+	return assets, close, nil
+}
+
+func pluralize(count int, label string) string {
+	if count == 1 {
+		return label
+	}
+	return fmt.Sprintf("%ss", label)
 }
