@@ -1,13 +1,19 @@
 package github
 
 import (
+	"bytes"
+	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"net/http"
 	"net/url"
 	"os"
 	"os/exec"
+	"path"
 	"path/filepath"
+	"sort"
 	"strings"
 	"time"
 
@@ -26,11 +32,27 @@ func NewClient(h string) *Client {
 }
 
 func NewClientWithHost(host *Host) *Client {
-	return &Client{host}
+	return &Client{Host: host}
 }
 
 type Client struct {
-	Host *Host
+	Host         *Host
+	cachedClient *simpleClient
+}
+
+type Gist struct {
+	Files       map[string]GistFile `json:"files"`
+	Description string              `json:"description,omitempty"`
+	Id          string              `json:"id,omitempty"`
+	Public      bool                `json:"public"`
+	HtmlUrl     string              `json:"html_url"`
+}
+
+type GistFile struct {
+	Type     string `json:"type,omitempty"`
+	Language string `json:"language,omitempty"`
+	Content  string `json:"content"`
+	RawUrl   string `json:"raw_url"`
 }
 
 func (client *Client) FetchPullRequests(project *Project, filterParams map[string]interface{}, limit int, filter func(*PullRequest) bool) (pulls []PullRequest, err error) {
@@ -41,21 +63,14 @@ func (client *Client) FetchPullRequests(project *Project, filterParams map[strin
 
 	path := fmt.Sprintf("repos/%s/%s/pulls?per_page=%d", project.Owner, project.Name, perPage(limit, 100))
 	if filterParams != nil {
-		query := url.Values{}
-		for key, value := range filterParams {
-			switch v := value.(type) {
-			case string:
-				query.Add(key, v)
-			}
-		}
-		path += "&" + query.Encode()
+		path = addQuery(path, filterParams)
 	}
 
 	pulls = []PullRequest{}
 	var res *simpleResponse
 
 	for path != "" {
-		res, err = api.Get(path)
+		res, err = api.GetFile(path, draftsType)
 		if err = checkStatus(200, "fetching pull requests", res, err); err != nil {
 			return
 		}
@@ -116,7 +131,7 @@ func (client *Client) CreatePullRequest(project *Project, params map[string]inte
 		return
 	}
 
-	res, err := api.PostJSON(fmt.Sprintf("repos/%s/%s/pulls", project.Owner, project.Name), params)
+	res, err := api.PostJSONPreview(fmt.Sprintf("repos/%s/%s/pulls", project.Owner, project.Name), params, draftsType)
 	if err = checkStatus(201, "creating pull request", res, err); err != nil {
 		if res != nil && res.StatusCode == 404 {
 			projectUrl := strings.SplitN(project.WebURL("", "", ""), "://", 2)[1]
@@ -160,13 +175,6 @@ func (client *Client) CommitPatch(project *Project, sha string) (patch io.ReadCl
 	return res.Body, nil
 }
 
-type Gist struct {
-	Files map[string]GistFile `json:"files"`
-}
-type GistFile struct {
-	RawUrl string `json:"raw_url"`
-}
-
 func (client *Client) GistPatch(id string) (patch io.ReadCloser, err error) {
 	api, err := client.simpleApi()
 	if err != nil {
@@ -373,26 +381,70 @@ func (client *Client) DeleteRelease(release *Release) (err error) {
 	return
 }
 
-func (client *Client) UploadReleaseAsset(release *Release, filename, label string) (asset *ReleaseAsset, err error) {
+type LocalAsset struct {
+	Name     string
+	Label    string
+	Contents io.Reader
+	Size     int64
+}
+
+func (client *Client) UploadReleaseAssets(release *Release, assets []LocalAsset) (doneAssets []*ReleaseAsset, err error) {
 	api, err := client.simpleApi()
 	if err != nil {
 		return
 	}
 
-	parts := strings.SplitN(release.UploadUrl, "{", 2)
-	uploadUrl := parts[0]
-	uploadUrl += "?name=" + url.QueryEscape(filepath.Base(filename))
-	if label != "" {
-		uploadUrl += "&label=" + url.QueryEscape(label)
+	idx := strings.Index(release.UploadUrl, "{")
+	uploadURL := release.UploadUrl[0:idx]
+
+	for _, asset := range assets {
+		for _, existingAsset := range release.Assets {
+			if existingAsset.Name == asset.Name {
+				if err = client.DeleteReleaseAsset(&existingAsset); err != nil {
+					return
+				}
+				break
+			}
+		}
+
+		params := map[string]interface{}{"name": filepath.Base(asset.Name)}
+		if asset.Label != "" {
+			params["label"] = asset.Label
+		}
+		uploadPath := addQuery(uploadURL, params)
+
+		var res *simpleResponse
+		attempts := 0
+		maxAttempts := 3
+		body := asset.Contents
+		for {
+			res, err = api.PostFile(uploadPath, body, asset.Size)
+			if err == nil && res.StatusCode >= 500 && res.StatusCode < 600 && attempts < maxAttempts {
+				attempts++
+				time.Sleep(time.Second * time.Duration(attempts))
+				var f *os.File
+				f, err = os.Open(asset.Name)
+				if err != nil {
+					return
+				}
+				defer f.Close()
+				body = f
+				continue
+			}
+			if err = checkStatus(201, "uploading release asset", res, err); err != nil {
+				return
+			}
+			break
+		}
+
+		newAsset := ReleaseAsset{}
+		err = res.Unmarshal(&newAsset)
+		if err != nil {
+			return
+		}
+		doneAssets = append(doneAssets, &newAsset)
 	}
 
-	res, err := api.PostFile(uploadUrl, filename)
-	if err = checkStatus(201, "uploading release asset", res, err); err != nil {
-		return
-	}
-
-	asset = &ReleaseAsset{}
-	err = res.Unmarshal(asset)
 	return
 }
 
@@ -460,6 +512,20 @@ func (client *Client) FetchCIStatus(project *Project, sha string) (status *CISta
 		return
 	}
 
+	sortStatuses := func() {
+		sort.Slice(status.Statuses, func(a, b int) bool {
+			sA := status.Statuses[a]
+			sB := status.Statuses[b]
+			cmp := strings.Compare(strings.ToLower(sA.Context), strings.ToLower(sB.Context))
+			if cmp == 0 {
+				return strings.Compare(sA.TargetUrl, sB.TargetUrl) < 0
+			} else {
+				return cmp < 0
+			}
+		})
+	}
+	sortStatuses()
+
 	res, err = api.GetFile(fmt.Sprintf("repos/%s/%s/commits/%s/check-runs", project.Owner, project.Name, sha), checksType)
 	if err == nil && (res.StatusCode == 403 || res.StatusCode == 404 || res.StatusCode == 422) {
 		return
@@ -486,6 +552,8 @@ func (client *Client) FetchCIStatus(project *Project, sha string) (status *CISta
 		status.Statuses = append(status.Statuses, checkStatus)
 	}
 
+	sortStatuses()
+
 	return
 }
 
@@ -544,6 +612,7 @@ type Issue struct {
 
 	MergeCommitSha      string `json:"merge_commit_sha"`
 	MaintainerCanModify bool   `json:"maintainer_can_modify"`
+	Draft               bool   `json:"draft"`
 
 	Comments  int          `json:"comments"`
 	Labels    []IssueLabel `json:"labels"`
@@ -622,14 +691,7 @@ func (client *Client) FetchIssues(project *Project, filterParams map[string]inte
 
 	path := fmt.Sprintf("repos/%s/%s/issues?per_page=%d", project.Owner, project.Name, perPage(limit, 100))
 	if filterParams != nil {
-		query := url.Values{}
-		for key, value := range filterParams {
-			switch v := value.(type) {
-			case string:
-				query.Add(key, v)
-			}
-		}
-		path += "&" + query.Encode()
+		path = addQuery(path, filterParams)
 	}
 
 	issues = []Issue{}
@@ -723,6 +785,18 @@ func (client *Client) UpdateIssue(project *Project, issueNumber int, params map[
 	return
 }
 
+type sortedLabels []IssueLabel
+
+func (s sortedLabels) Len() int {
+	return len(s)
+}
+func (s sortedLabels) Swap(i, j int) {
+	s[i], s[j] = s[j], s[i]
+}
+func (s sortedLabels) Less(i, j int) bool {
+	return strings.Compare(strings.ToLower(s[i].Name), strings.ToLower(s[j].Name)) < 0
+}
+
 func (client *Client) FetchLabels(project *Project) (labels []IssueLabel, err error) {
 	api, err := client.simpleApi()
 	if err != nil {
@@ -748,6 +822,8 @@ func (client *Client) FetchLabels(project *Project) (labels []IssueLabel, err er
 		labels = append(labels, labelsPage...)
 	}
 
+	sort.Sort(sortedLabels(labels))
+
 	return
 }
 
@@ -779,6 +855,78 @@ func (client *Client) FetchMilestones(project *Project) (milestones []Milestone,
 	return
 }
 
+func (client *Client) GenericAPIRequest(method, path string, data interface{}, headers map[string]string, ttl int) (*simpleResponse, error) {
+	api, err := client.simpleApi()
+	if err != nil {
+		return nil, err
+	}
+	api.CacheTTL = ttl
+
+	var body io.Reader
+	switch d := data.(type) {
+	case map[string]interface{}:
+		if method == "GET" {
+			path = addQuery(path, d)
+		} else if len(d) > 0 {
+			json, err := json.Marshal(d)
+			if err != nil {
+				return nil, err
+			}
+			body = bytes.NewBuffer(json)
+		}
+	case io.Reader:
+		body = d
+	}
+
+	return api.performRequest(method, path, body, func(req *http.Request) {
+		if body != nil {
+			req.Header.Set("Content-Type", "application/json; charset=utf-8")
+		}
+		for key, value := range headers {
+			req.Header.Set(key, value)
+		}
+	})
+}
+
+// GraphQL facilitates performing a GraphQL request and parsing the response
+func (client *Client) GraphQL(query string, variables interface{}, data interface{}) error {
+	api, err := client.simpleApi()
+	if err != nil {
+		return err
+	}
+
+	payload := map[string]interface{}{
+		"query":     query,
+		"variables": variables,
+	}
+	resp, err := api.PostJSON("graphql", payload)
+	if err = checkStatus(200, "performing GraphQL", resp, err); err != nil {
+		return err
+	}
+
+	responseData := struct {
+		Data   interface{}
+		Errors []struct {
+			Message string
+		}
+	}{
+		Data: data,
+	}
+	err = resp.Unmarshal(&responseData)
+	if err != nil {
+		return err
+	}
+
+	if len(responseData.Errors) > 0 {
+		messages := []string{}
+		for _, e := range responseData.Errors {
+			messages = append(messages, e.Message)
+		}
+		return fmt.Errorf("API error: %s", strings.Join(messages, "; "))
+	}
+	return nil
+}
+
 func (client *Client) CurrentUser() (user *User, err error) {
 	api, err := client.simpleApi()
 	if err != nil {
@@ -819,7 +967,7 @@ func (client *Client) FindOrCreateToken(user, password, twoFactorCode string) (t
 	}
 
 	params := map[string]interface{}{
-		"scopes":   []string{"repo"},
+		"scopes":   []string{"repo", "gist"},
 		"note_url": OAuthAppURL,
 	}
 
@@ -867,14 +1015,15 @@ func (client *Client) FindOrCreateToken(user, password, twoFactorCode string) (t
 	return
 }
 
-func (client *Client) ensureAccessToken() (err error) {
+func (client *Client) ensureAccessToken() error {
 	if client.Host.AccessToken == "" {
 		host, err := CurrentConfig().PromptForHost(client.Host.Host)
-		if err == nil {
-			client.Host = host
+		if err != nil {
+			return err
 		}
+		client.Host = host
 	}
-	return
+	return nil
 }
 
 func (client *Client) simpleApi() (c *simpleClient, err error) {
@@ -883,10 +1032,24 @@ func (client *Client) simpleApi() (c *simpleClient, err error) {
 		return
 	}
 
+	if client.cachedClient != nil {
+		c = client.cachedClient
+		return
+	}
+
 	c = client.apiClient()
 	c.PrepareRequest = func(req *http.Request) {
-		req.Header.Set("Authorization", "token "+client.Host.AccessToken)
+		clientDomain := normalizeHost(client.Host.Host)
+		if strings.HasPrefix(clientDomain, "api.github.") {
+			clientDomain = strings.TrimPrefix(clientDomain, "api.")
+		}
+		requestHost := strings.ToLower(req.URL.Host)
+		if requestHost == clientDomain || strings.HasSuffix(requestHost, "."+clientDomain) {
+			req.Header.Set("Authorization", "token "+client.Host.AccessToken)
+		}
 	}
+
+	client.cachedClient = c
 	return
 }
 
@@ -905,13 +1068,69 @@ func (client *Client) apiClient() *simpleClient {
 }
 
 func (client *Client) absolute(host string) *url.URL {
-	u, _ := url.Parse("https://" + host + "/")
-	if client.Host != nil && client.Host.Protocol != "" {
+	u, err := url.Parse("https://" + host + "/")
+	if err != nil {
+		panic(err)
+	} else if client.Host != nil && client.Host.Protocol != "" {
 		u.Scheme = client.Host.Protocol
 	}
 	return u
 }
 
+func (client *Client) FetchGist(id string) (gist *Gist, err error) {
+	api, err := client.simpleApi()
+	if err != nil {
+		return
+	}
+
+	response, err := api.Get(fmt.Sprintf("gists/%s", id))
+	if err = checkStatus(200, "getting gist", response, err); err != nil {
+		return
+	}
+
+	response.Unmarshal(&gist)
+	return
+}
+
+func (client *Client) CreateGist(filenames []string, public bool) (gist *Gist, err error) {
+	api, err := client.simpleApi()
+	if err != nil {
+		return
+	}
+	files := map[string]GistFile{}
+	var basename string
+	var content []byte
+	var gf GistFile
+
+	for _, file := range filenames {
+		if file == "-" {
+			content, err = ioutil.ReadAll(os.Stdin)
+			basename = "gistfile1.txt"
+		} else {
+			content, err = ioutil.ReadFile(file)
+			basename = path.Base(file)
+		}
+		if err != nil {
+			return
+		}
+		gf = GistFile{Content: string(content)}
+		files[basename] = gf
+	}
+
+	g := Gist{
+		Files:  files,
+		Public: public,
+	}
+
+	res, err := api.PostJSON("gists", &g)
+	if err = checkStatus(201, "creating gist", res, err); err != nil {
+		return
+	}
+
+	err = res.Unmarshal(&gist)
+	return
+}
+
 func normalizeHost(host string) string {
 	if host == "" {
 		return GitHubHost
@@ -924,65 +1143,165 @@ func normalizeHost(host string) string {
 	}
 }
 
-func checkStatus(expectedStatus int, action string, response *simpleResponse, err error) error {
-	if err != nil {
-		return fmt.Errorf("Error %s: %s", action, err.Error())
-	} else if response.StatusCode != expectedStatus {
-		errInfo, err := response.ErrorInfo()
-		if err == nil {
-			return FormatError(action, errInfo)
-		} else {
-			return fmt.Errorf("Error %s: %s (HTTP %d)", action, err.Error(), response.StatusCode)
-		}
-	} else {
-		return nil
+func reverseNormalizeHost(host string) string {
+	switch host {
+	case "api.github.com":
+		return GitHubHost
+	case "api.github.localhost":
+		return "github.localhost"
+	default:
+		return host
 	}
 }
 
-func FormatError(action string, err error) (ee error) {
-	switch e := err.(type) {
-	default:
-		ee = err
-	case *errorInfo:
-		statusCode := e.Response.StatusCode
-		var reason string
-		if s := strings.SplitN(e.Response.Status, " ", 2); len(s) >= 2 {
-			reason = strings.TrimSpace(s[1])
+func checkStatus(expectedStatus int, action string, response *simpleResponse, err error) error {
+	if err != nil {
+		errStr := err.Error()
+		if urlErr, isURLErr := err.(*url.Error); isURLErr {
+			errStr = fmt.Sprintf("%s %s: %s", urlErr.Op, urlErr.URL, urlErr.Err)
 		}
-
-		errStr := fmt.Sprintf("Error %s: %s (HTTP %d)", action, reason, statusCode)
-
-		var errorSentences []string
-		for _, err := range e.Errors {
-			switch err.Code {
-			case "custom":
-				errorSentences = append(errorSentences, err.Message)
-			case "missing_field":
-				errorSentences = append(errorSentences, fmt.Sprintf("Missing field: \"%s\"", err.Field))
-			case "already_exists":
-				errorSentences = append(errorSentences, fmt.Sprintf("Duplicate value for \"%s\"", err.Field))
-			case "invalid":
-				errorSentences = append(errorSentences, fmt.Sprintf("Invalid value for \"%s\"", err.Field))
-			case "unauthorized":
-				errorSentences = append(errorSentences, fmt.Sprintf("Not allowed to change field \"%s\"", err.Field))
-			}
+		return fmt.Errorf("Error %s: %s", action, errStr)
+	} else if response.StatusCode != expectedStatus {
+		errInfo, err := response.ErrorInfo()
+		if err != nil {
+			return fmt.Errorf("Error %s: %s (HTTP %d)", action, err.Error(), response.StatusCode)
 		}
+		return FormatError(action, errInfo)
+	}
+	return nil
+}
 
-		var errorMessage string
-		if len(errorSentences) > 0 {
-			errorMessage = strings.Join(errorSentences, "\n")
-		} else {
-			errorMessage = e.Message
-		}
+// FormatError annotates an HTTP response error with user-friendly messages
+func FormatError(action string, err error) error {
+	if e, ok := err.(*errorInfo); ok {
+		return formatError(action, e)
+	}
+	return err
+}
 
-		if errorMessage != "" {
-			errStr = fmt.Sprintf("%s\n%s", errStr, errorMessage)
-		}
-
-		ee = fmt.Errorf(errStr)
+func formatError(action string, e *errorInfo) error {
+	var reason string
+	if s := strings.SplitN(e.Response.Status, " ", 2); len(s) >= 2 {
+		reason = strings.TrimSpace(s[1])
 	}
 
-	return
+	errStr := fmt.Sprintf("Error %s: %s (HTTP %d)", action, reason, e.Response.StatusCode)
+
+	var errorSentences []string
+	for _, err := range e.Errors {
+		switch err.Code {
+		case "custom":
+			errorSentences = append(errorSentences, err.Message)
+		case "missing_field":
+			errorSentences = append(errorSentences, fmt.Sprintf("Missing field: \"%s\"", err.Field))
+		case "already_exists":
+			errorSentences = append(errorSentences, fmt.Sprintf("Duplicate value for \"%s\"", err.Field))
+		case "invalid":
+			errorSentences = append(errorSentences, fmt.Sprintf("Invalid value for \"%s\"", err.Field))
+		case "unauthorized":
+			errorSentences = append(errorSentences, fmt.Sprintf("Not allowed to change field \"%s\"", err.Field))
+		}
+	}
+
+	var errorMessage string
+	if len(errorSentences) > 0 {
+		errorMessage = strings.Join(errorSentences, "\n")
+	} else {
+		errorMessage = e.Message
+		if action == "getting current user" && e.Message == "Resource not accessible by integration" {
+			errorMessage = errorMessage + "\nYou must specify GITHUB_USER via environment variable."
+		}
+	}
+	if errorMessage != "" {
+		errStr = fmt.Sprintf("%s\n%s", errStr, errorMessage)
+	}
+
+	if ssoErr := ValidateGitHubSSO(e.Response); ssoErr != nil {
+		return fmt.Errorf("%s\n%s", errStr, ssoErr)
+	}
+
+	if scopeErr := ValidateSufficientOAuthScopes(e.Response); scopeErr != nil {
+		return fmt.Errorf("%s\n%s", errStr, scopeErr)
+	}
+
+	return errors.New(errStr)
+}
+
+// ValidateGitHubSSO checks for the challenge via `X-Github-Sso` header
+func ValidateGitHubSSO(res *http.Response) error {
+	if res.StatusCode != 403 {
+		return nil
+	}
+
+	sso := res.Header.Get("X-Github-Sso")
+	if !strings.HasPrefix(sso, "required; url=") {
+		return nil
+	}
+
+	url := sso[strings.IndexByte(sso, '=')+1:]
+	return fmt.Errorf("You must authorize your token to access this organization:\n%s", url)
+}
+
+// ValidateSufficientOAuthScopes warns about insufficient OAuth scopes
+func ValidateSufficientOAuthScopes(res *http.Response) error {
+	if res.StatusCode != 404 && res.StatusCode != 403 {
+		return nil
+	}
+
+	needScopes := newScopeSet(res.Header.Get("X-Accepted-Oauth-Scopes"))
+	if len(needScopes) == 0 && isGistWrite(res.Request) {
+		// compensate for a GitHub bug: gist APIs omit proper `X-Accepted-Oauth-Scopes` in responses
+		needScopes = newScopeSet("gist")
+	}
+
+	haveScopes := newScopeSet(res.Header.Get("X-Oauth-Scopes"))
+	if len(needScopes) == 0 || needScopes.Intersects(haveScopes) {
+		return nil
+	}
+
+	return fmt.Errorf("Your access token may have insufficient scopes. Visit %s://%s/settings/tokens\n"+
+		"to edit the 'hub' token and enable one of the following scopes: %s",
+		res.Request.URL.Scheme,
+		reverseNormalizeHost(res.Request.Host),
+		needScopes)
+}
+
+func isGistWrite(req *http.Request) bool {
+	if req.Method == "GET" {
+		return false
+	}
+	path := strings.TrimPrefix(req.URL.Path, "/v3")
+	return strings.HasPrefix(path, "/gists")
+}
+
+type scopeSet map[string]struct{}
+
+func (s scopeSet) String() string {
+	scopes := make([]string, 0, len(s))
+	for scope := range s {
+		scopes = append(scopes, scope)
+	}
+	sort.Sort(sort.StringSlice(scopes))
+	return strings.Join(scopes, ", ")
+}
+
+func (s scopeSet) Intersects(other scopeSet) bool {
+	for scope := range s {
+		if _, found := other[scope]; found {
+			return true
+		}
+	}
+	return false
+}
+
+func newScopeSet(s string) scopeSet {
+	scopes := scopeSet{}
+	for _, s := range strings.SplitN(s, ",", -1) {
+		if s = strings.TrimSpace(s); s != "" {
+			scopes[s] = struct{}{}
+		}
+	}
+	return scopes
 }
 
 func authTokenNote(num int) (string, error) {
@@ -1022,3 +1341,29 @@ func perPage(limit, max int) int {
 	}
 	return max
 }
+
+func addQuery(path string, params map[string]interface{}) string {
+	if len(params) == 0 {
+		return path
+	}
+
+	query := url.Values{}
+	for key, value := range params {
+		switch v := value.(type) {
+		case string:
+			query.Add(key, v)
+		case nil:
+			query.Add(key, "")
+		case int:
+			query.Add(key, fmt.Sprintf("%d", v))
+		case bool:
+			query.Add(key, fmt.Sprintf("%v", v))
+		}
+	}
+
+	sep := "?"
+	if strings.Contains(path, sep) {
+		sep = "&"
+	}
+	return path + sep + query.Encode()
+}
