@@ -113,14 +113,16 @@ func (r *GitHubRepo) MasterBranch() *Branch {
 }
 
 func (r *GitHubRepo) DefaultBranch(remote *Remote) *Branch {
-	var name string
+	b := Branch{
+		Repo: r,
+		Name: "refs/heads/master",
+	}
 	if remote != nil {
-		name, _ = git.BranchAtRef("refs", "remotes", remote.Name, "HEAD")
+		if name, err := git.SymbolicRef(fmt.Sprintf("refs/remotes/%s/HEAD", remote.Name)); err == nil {
+			b.Name = name
+		}
 	}
-	if name == "" {
-		name = "refs/heads/master"
-	}
-	return &Branch{r, name}
+	return &b
 }
 
 func (r *GitHubRepo) RemoteBranchAndProject(owner string, preferUpstream bool) (branch *Branch, project *Project, err error) {
@@ -140,22 +142,65 @@ func (r *GitHubRepo) RemoteBranchAndProject(owner string, preferUpstream bool) (
 		return
 	}
 
-	if project != nil {
-		branch = branch.PushTarget(owner, preferUpstream)
-		if branch != nil && branch.IsRemote() {
-			remote, e := r.RemoteByName(branch.RemoteName())
+	if project == nil {
+		return
+	}
+
+	pushDefault, _ := git.Config("push.default")
+	if pushDefault == "upstream" || pushDefault == "tracking" {
+		upstream, e := branch.Upstream()
+		if e == nil && upstream.IsRemote() {
+			remote, e := r.RemoteByName(upstream.RemoteName())
 			if e == nil {
-				project, err = remote.Project()
-				if err != nil {
+				p, e := remote.Project()
+				if e == nil {
+					branch = upstream
+					project = p
 					return
 				}
 			}
 		}
 	}
 
+	shortName := branch.ShortName()
+	remotes := r.remotesForPublish(owner)
+	if preferUpstream {
+		// reverse the remote lookup order; see OriginNamesInLookupOrder
+		remotesInOrder := []Remote{}
+		for i := len(remotes) - 1; i >= 0; i-- {
+			remotesInOrder = append(remotesInOrder, remotes[i])
+		}
+		remotes = remotesInOrder
+	}
+
+	for _, remote := range remotes {
+		p, e := remote.Project()
+		if e != nil {
+			continue
+		}
+		// NOTE: this is similar RemoteForBranch
+		if git.HasFile("refs", "remotes", remote.Name, shortName) {
+			name := fmt.Sprintf("refs/remotes/%s/%s", remote.Name, shortName)
+			branch = &Branch{r, name}
+			project = p
+			return
+		}
+	}
+
+	branch = nil
 	return
 }
 
+func (r *GitHubRepo) RemoteForBranch(branch *Branch, owner string) *Remote {
+	branchName := branch.ShortName()
+	for _, remote := range r.remotesForPublish(owner) {
+		if git.HasFile("refs", "remotes", remote.Name, branchName) {
+			return &remote
+		}
+	}
+	return nil
+}
+
 func (r *GitHubRepo) RemoteForRepo(repo *Repository) (*Remote, error) {
 	if err := r.loadRemotes(); err != nil {
 		return nil, err
