@@ -11,6 +11,7 @@ import (
 	"time"
 
 	"github.com/github/hub/ui"
+	"github.com/kballard/go-shellquote"
 )
 
 var timeNow = time.Now
@@ -30,13 +31,15 @@ func BrowserLauncher() ([]string, error) {
 	browser := os.Getenv("BROWSER")
 	if browser == "" {
 		browser = searchBrowserLauncher(runtime.GOOS)
+	} else {
+		browser = os.ExpandEnv(browser)
 	}
 
 	if browser == "" {
 		return nil, errors.New("Please set $BROWSER to a web launcher")
 	}
 
-	return strings.Split(browser, " "), nil
+	return shellquote.Split(browser)
 }
 
 func searchBrowserLauncher(goos string) (browser string) {
@@ -61,7 +64,7 @@ func searchBrowserLauncher(goos string) (browser string) {
 }
 
 func CommandPath(cmd string) (string, error) {
-	if runtime.GOOS == "windows" {
+	if runtime.GOOS == "windows" && !strings.HasSuffix(cmd, ".exe") {
 		cmd = cmd + ".exe"
 	}
 
@@ -78,10 +81,6 @@ func CommandPath(cmd string) (string, error) {
 	return filepath.EvalSymlinks(path)
 }
 
-func IsOption(confirm, short, long string) bool {
-	return strings.EqualFold(confirm, short) || strings.EqualFold(confirm, long)
-}
-
 func TimeAgo(t time.Time) string {
 	duration := timeNow().Sub(t)
 	minutes := duration.Minutes()
